package com.tzld.piaoquan.recommend.server.service.rank.strategy;


import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.service.rank.RankResult;
import com.tzld.piaoquan.recommend.server.service.rank.RankService;
import com.tzld.piaoquan.recommend.server.service.rank.extractor.RankExtractorFeature;
import com.tzld.piaoquan.recommend.server.service.rank.processor.RankProcessorDensity;
import com.tzld.piaoquan.recommend.server.service.rank.processor.RankProcessorTagFilter;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhangbo
 * @desc 带密度控制的后处理 排序实验
 */
@Service
@Slf4j
public class RankStrategy4Density extends RankService {
    @ApolloJsonValue("${RankStrategy4DensityFilter:}")
    private Map<String,Map<String, Map<String, String>>> filterRules;
    @Override
    public RankResult mergeAndSort(RankParam param, List<Video> rovRecallRank, List<Video> flowPoolRank) {

        if (CollectionUtils.isEmpty(rovRecallRank)) {
            if (param.getSize() < flowPoolRank.size()) {
                return new RankResult(flowPoolRank.subList(0, param.getSize()));
            } else {
                return new RankResult(flowPoolRank);
            }
        }
        // 1 读取多样性密度控制规则------------------
//        String appType = String.valueOf(param.getAppType());
//        String ruleStr = this.redisTemplate.opsForValue().get("TAGS_FILTER_RULE_V1_JSON");
//        Map<String, Integer> densityRules = new HashMap<>();
//        if (ruleStr != null){
//            Map<String, Map<String, Object>> ruleOrigin = JSONUtils.fromJson(ruleStr,
//                    new TypeToken<Map<String, Map<String, Object>>>() {},
//                    Collections.emptyMap());
//            for (Map.Entry<String, Map<String, Object>> entry : ruleOrigin.entrySet()){
//                String k = entry.getKey();
//                if (!entry.getValue().containsKey(appType)){
//                    continue;
//                }
//                JSONObject jb = (JSONObject) entry.getValue().get(appType);
//                try{
//                    if (jb.containsKey("density") && jb.get("density") instanceof Integer){
//                        densityRules.put(k, jb.getInteger("density"));
//                    }
//                }catch (Exception e){
//                    log.error("parse densityRules is wrong:", e);
//                }
//            }
//        }
        Map<String, Integer> densityRules = new HashMap<>();
        String abCode = param.getAbCode();
        if (this.filterRules != null && this.filterRules.containsKey(abCode)){
            Map<String, Map<String, String>> rule = this.filterRules.get(abCode);
            for (Map.Entry<String, Map<String, String>> entry : rule.entrySet()){
                String key = entry.getKey();
                Map<String, String> value = entry.getValue();
                if (value.containsKey("density")){
                    densityRules.put(key, Integer.valueOf(value.get("density")));
                }
            }
        }


        // 2 读取video的tags------------------
        List<Long> videoIds = new ArrayList<>();
        for (Video v : rovRecallRank) {
            videoIds.add(v.getVideoId());
        }
        for (Video v : flowPoolRank) {
            videoIds.add(v.getVideoId());
        }
        Map<Long, List<String>> videoTagDict = RankExtractorFeature.getVideoTags(this.redisTemplate, videoIds);
        for (Video v : rovRecallRank) {
            v.setTags(videoTagDict.getOrDefault(v.getVideoId(), new ArrayList<>()));
        }
        for (Video v : flowPoolRank) {
            v.setTags(videoTagDict.getOrDefault(v.getVideoId(), new ArrayList<>()));
        }

        // 3 读取过滤规则，根据tag和过滤规则进行过滤------------------
        if (this.filterRules != null && this.filterRules.containsKey(abCode)){
            Map<String, Map<String, String>> rule = this.filterRules.get(abCode);
            RankProcessorTagFilter.tagFitlter(rovRecallRank, flowPoolRank, rule);
        }

        // 4 流量池按比例强插---------------------
        List<Video> result = new ArrayList<>();
        for (int i = 0; i < param.getTopK() && i < rovRecallRank.size(); i++) {
            result.add(rovRecallRank.get(i));
        }

        double flowPoolP = getFlowPoolP(param);
        int flowPoolIndex = 0;
        int rovPoolIndex = param.getTopK();

        for (int i = 0; i < param.getSize() - param.getTopK(); i++) {
            double rand = RandomUtils.nextDouble(0, 1);
            log.info("rand={}, flowPoolP={}", rand, flowPoolP);
            if (rand < flowPoolP) {
                if (flowPoolIndex < flowPoolRank.size()) {
                    result.add(flowPoolRank.get(flowPoolIndex++));
                } else {
                    break;
                }
            } else {
                if (rovPoolIndex < rovRecallRank.size()) {
                    result.add(rovRecallRank.get(rovPoolIndex++));
                } else {
                    break;
                }
            }
        }
        if (rovPoolIndex >= rovRecallRank.size()) {
            for (int i = flowPoolIndex; i < flowPoolRank.size() && result.size() < param.getSize(); i++) {
                result.add(flowPoolRank.get(i));
            }
        }
        if (flowPoolIndex >= flowPoolRank.size()) {
            for (int i = rovPoolIndex; i < rovRecallRank.size() && result.size() < param.getSize(); i++) {
                result.add(rovRecallRank.get(i));
            }
        }

        // 3 进行密度控制------------------
        Set<Long> videosSet = result.stream().map(r-> r.getVideoId()).collect(Collectors.toSet());
        List<Video> rovRecallRankNew = rovRecallRank.stream().filter(r -> !videosSet.contains(r.getVideoId())).collect(Collectors.toList());
        List<Video> flowPoolRankNew = flowPoolRank.stream().filter(r -> !videosSet.contains(r.getVideoId())).collect(Collectors.toList());
        List<Video> resultWithDnsity = RankProcessorDensity.mergeDensityControl(result,
                rovRecallRankNew, flowPoolRankNew, densityRules);
//        log.info("zhangbo22 old={}, new={}", JSONUtils.toJson(result),
//                    JSONUtils.toJson(resultWithDnsity));
        return new RankResult(resultWithDnsity);
    }



//    public Video getTestVideo(Long id, String s){
//        Video a1 = new Video();
//        a1.setVideoId(id);
//        a1.setFlowPool(s);
//        return a1;
//    }



}
